@@ -21,42 +21,66 @@ class Spawn(pexpect.spawn):
 
 
 class BondException(exceptions.IOError):
-    pass
+    def __init__(self, lang, error):
+        self.lang = lang
+        super(BondException, self).__init__(error)
 
-class StateException(BondException):
-    pass
+    def __str__(self):
+        return "BondException[{lang}]: {msg}".format(lang=self.lang, msg=self.message)
+
+class SerializationException(BondException):
+    def __init__(self, lang, error, side):
+        self.side = side
+        super(SerializationException, self).__init__(lang, error)
+
+    def __str__(self):
+        return "SerializationException[{lang}, {side}]: {msg}".format(
+            lang=self.lang, side=self.side, msg=self.message)
 
 class RemoteException(BondException):
-    pass
+    def __init__(self, lang, error, data):
+        self.data = data
+        super(RemoteException, self).__init__(lang, error)
+
+    def __str__(self):
+        return "RemoteException[{lang}]: {msg}".format(lang=self.lang, msg=self.message)
 
 
 class Bond(object):
     LANG = '<unknown>'
 
     def __init__(self, proc):
-        self.channels = {'STDOUT': sys.stdout}
+        self.channels = {'STDOUT': sys.stdout, 'STDERR': sys.stderr}
         self.bindings = {}
 
         self._proc = proc
         try:
             self._proc.expect("READY\r\n")
         except pexpect.ExceptionPexpect as e:
-            raise StateException('unknown "{lang}" interpreter state'.format(lang=self.LANG))
+            raise BondException(self.LANG, 'unknown interpreter state')
 
 
-    def _loads(self, string):
+    def loads(self, string):
         return json.loads(string)
 
-    def _dumps(self, *args):
+    def dumps(self, *args):
         return json.dumps(*args)
 
 
+    def _dumps(self, *args):
+        try:
+            return self.dumps(*args)
+        except Exception as e:
+            raise SerializationException(self.LANG, str(e), 'local')
+        return ret
+
+
     def _repl(self):
         while self._proc.expect("(\S*)(?: ([^\r\n]+))?\r\n") == 0:
             cmd = str(self._proc.match.group(1))
             args = self._proc.match.group(2)
             if args is not None:
-                args = self._loads(args)
+                args = self.loads(args)
 
             # interpret the serial protocol
             if cmd == "OUTPUT":
@@ -67,22 +91,24 @@ class Bond(object):
                 ret = self._dumps(ret) if ret else None
                 self._proc.sendline('RETURN {ret}'.format(ret=ret))
                 continue
+            elif cmd == "EXCEPT":
+                raise RemoteException(self.LANG, str(args), args)
             elif cmd == "ERROR":
-                raise RemoteException('{lang}: {error}'.format(lang=self.LANG, error=str(args)))
+                raise SerializationException(self.LANG, str(args), 'remote')
             elif cmd == "RETURN":
                 return args
 
-            raise StateException('unknown "{lang}" interpreter state'.format(lang=self.LANG))
+            raise BondException(self.LANG, 'unknown interpreter state')
 
 
     def eval(self, code):
-        '''Evaluate a single statement in the interpreter and return its value'''
+        '''Evaluate and return the value of a single statement of code in the interpreter.'''
         code = self._dumps(code)
         self._proc.sendline('EVAL {code}'.format(code=code))
         return self._repl()
 
     def eval_block(self, code):
-        '''Evaluate a "code" block inside the interpreter'''
+        '''Evaluate a "code" block inside the interpreter. Nothing is returned.'''
         code = self._dumps(code)
         self._proc.sendline('EVAL_BLOCK {code}'.format(code=code))
         return self._repl()
@@ -97,8 +123,11 @@ class Bond(object):
         '''Terminate the underlying interpreter'''
         self._proc.sendeof()
 
-    def export(self, func, name):
-        '''Export a local function "func" to be callable in the interpreter as "name"'''
+    def export(self, func, name=None):
+        '''Export a local function "func" to be callable in the interpreter as "name".
+        If "name" is not specified, use the local function name directly.'''
+        if name is None:
+            name = func.__name__
         self.bindings[name] = func
         self._proc.sendline('EXPORT {name}'.format(name=self._dumps(name)))
         return self._repl()
@@ -121,10 +150,10 @@ class Bond(object):
 def interact(bond, prompt=None):
     '''Start an interactive session with "bond"
 
-    If "prompt" is not specified, use the language name of the bond.  By
-    default, all input lines are executed with bond.eval_block(), which might
-    not output the result of the expression. If "!" is pre-pended, execute a
-    single statement with bond.eval() instead.'''
+    If "prompt" is not specified, use the language name of the bond. By
+    default, all input lines are executed with bond.eval_block().  If "!" is
+    pre-pended, execute a single statement with bond.eval() and print it's
+    return value'''
 
     if prompt is None:
         prompt = "{lang}> ".format(lang=bond.LANG)
@@ -145,7 +174,7 @@ def interact(bond, prompt=None):
                 ret = bond.eval(line[1:])
             else:
                 ret = bond.eval_block(line)
-        except RemoteException as e:
+        except (RemoteException, SerializationException) as e:
             ret = e
         if ret is not None:
             print(ret)
